#include <bcm2835.h>
#include <stdio.h>
#include "ch1115.h"
#define WIDTH 128
#define HEIGHT 64
#define PAGES 8

#define RST 25
#define DC  24
char buffer[1024];

void command(char cmd) {
    bcm2835_gpio_write(DC, LOW);
    bcm2835_spi_transfer(cmd);
}

void CH1115_begin()
{
    bcm2835_gpio_fsel(RST, BCM2835_GPIO_FSEL_OUTP);
    bcm2835_gpio_fsel(DC, BCM2835_GPIO_FSEL_OUTP);

    bcm2835_spi_begin();
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);     //The default
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                  //The default
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_2048);  //The default
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                     //The default
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);     //the default

    bcm2835_gpio_write(RST, HIGH);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, LOW);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, HIGH);

    command(0xAE); /*display off*/ 
    command(0x00); /*set lower column address*/ 
    command(0x10); /*set higher column address*/
    command(0xB0); /*set page address*/ 
    command(0x40); /*set display start lines*/ 
    command(0x81); /*contract control*/ 
    command(0x80); /*4d*/ 
    command(0x82); /* iref resistor set and adjust ISEG*/ 
    command(0x00); 
    command(0xA0); /*set segment remap 0xA0*/ 
    command(0xA2); /*set seg pads hardware configuration*/ 
    command(0xA4); /*Disable Entire Display On (0xA4/0xA5)*/ 
    command(0xA6); /*normal / reverse*/ 
    command(0xA8); /*multiplex ratio*/ 
    command(0x3F); /*duty = 1/64*/ 
    command(0xC0); /*Com scan direction 0XC0*/ 
    command(0xD3); /*set display offset*/ 
    command(0x00); /* */ 
    command(0xD5); /*set osc division*/ 
    command(0xa0); 
    command(0xD9); /*set pre-charge period*/ 
    command(0x22); 
    command(0xdb); /*set vcomh*/ 
    command(0x40); 
    command(0x31); /* Set pump 7.4v */ 
    command(0xad); /*set charge pump enable*/ 
    command(0x8b); /*Set DC-DC enable (0x8a=disable; 0x8b=enable) */ 
    
    command(0xAF);

}
void CH1115_clear()
{
    int i;
    for(i = 0; i < sizeof(buffer); i++)
    {
        buffer[i] = 0;
    }
}
void CH1115_pixel(int x, int y, char color)
{
    if(x > WIDTH || y > HEIGHT)return ;
    if(color)
        buffer[x+(y/8)*WIDTH] |= 1<<(y%8);
    else
        buffer[x+(y/8)*WIDTH] &= ~(1<<(y%8));
}
void CH1115_char1616(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 32; i ++) {
        chTemp = Font1612[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            CH1115_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 16) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}

void CH1115_char3216(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 64; i ++) {
        chTemp = Font3216[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            CH1115_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 32) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void CH1115_char(unsigned char x, unsigned char y, char acsii, char size, char mode)
{
    unsigned char i, j, y0=y;
    char temp;
    unsigned char ch = acsii - ' ';
    for(i = 0;i<size;i++) {
        if(size == 12)
        {
            if(mode)temp=Font1206[ch][i];
            else temp = ~Font1206[ch][i];
        }
        else 
        {            
            if(mode)temp=Font1608[ch][i];
            else temp = ~Font1608[ch][i];
        }
        for(j =0;j<8;j++)
        {
            if(temp & 0x80) CH1115_pixel(x, y, 1);
            else CH1115_pixel(x, y, 0);
            temp <<=1;
            y++;
            if((y-y0)==size)
            {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void CH1115_string(uint8_t x, uint8_t y, const char *pString, uint8_t Size, uint8_t Mode)
{
    while (*pString != '\0') {       
        if (x > (WIDTH - Size / 2)) {
            x = 0;
            y += Size;
            if (y > (HEIGHT - Size)) {
                y = x = 0;
            }
        }
        
        CH1115_char(x, y, *pString, Size, Mode);
        x += Size / 2;
        pString ++;
    }
}

void CH1115_bitmap(unsigned char x, unsigned char y, const unsigned char *pBmp, unsigned char chWidth, unsigned char chHeight)
{
    unsigned char i, j, byteWidth = (chWidth + 7)/8;
    for(j = 0; j < chHeight; j++) {
        for(i = 0;i <chWidth;i ++) {
            if(*(pBmp + j*byteWidth + i/8) & (128 >> (i & 7))) {
                CH1115_pixel(x+i, y+j, 1);
            }
        }
    }        
}

void CH1115_display()
{   int i;
    uint8_t page;
    char *pBuf = (char *)buffer;
    
    for (page = 0; page < 8; page++) {  
        // set page address 
        command(0xB0 + page);
        // set low column address
        command(0x00); 
        // set high column address 
        command(0x10); 
        // write data 
        for(i = 0; i< WIDTH; i++ ) {
    	    bcm2835_gpio_write(DC, HIGH);
    	    bcm2835_spi_transfer(pBuf[i+page*WIDTH]);
        }
    }
}